/*	$NetBSD$	*/

/*
 * Copyright (c) 2001 Minoura Makoto.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Synaptics TouchPad variable definitions.
 */

#define STP_ZTHRESH_DEFAULT	30 /* should be customizable */
#define STP_HISTORY_SIZE	7
#define STP_AVG_NSAMPLES	3
#define STP_LOOK_AT		(STP_HISTORY_SIZE-STP_AVG_NSAMPLES)
#define STP_TAP_MAX_DURATION	3

typedef struct stp_identifier {
	int		si_info_major;
	int		si_info_minor;
	int		si_info_modelcode;
	u_int16_t	si_capabilities;
	u_int32_t	si_model_id;
} stp_identifier;
	
typedef int (*stp_identify_func) __P((void *, stp_identifier *));
typedef int (*stp_getmode_func) __P((void *, u_int8_t *));
typedef int (*stp_setmode_func) __P((void *, u_int8_t));

typedef struct stp_abspacket {
	int sa_lbutton, sa_rbutton, sa_finger, sa_gesture;
	int sa_x, sa_y, sa_z;
} stp_abspacket;

typedef enum stp_pad_region {
	LEFT   = 1,
	RIGHT  = 2,
	TOP    = 4,
	BOTTOM = 8,
} stp_pad_region;

struct stp_softc {
	struct device	 	sc_dev;

	stp_identifier		sc_ident;
	stp_abspacket		sc_history[STP_HISTORY_SIZE];
	int			sc_seqno;
	enum {
		STP_IDLE,
		STP_XYMOVE,
		STP_TAPPING,
		STP_DRAGGING,
		STP_SCROLLING,
	}			sc_status;
	int			sc_x, sc_y;
	u_int			sc_buttons;
	u_int			sc_zthresh;
	int			sc_scraxis, sc_scrz;

	struct device		*sc_wsmousedev;
	const struct wsmouse_accessops *sc_wsaccess;

	/* methods */
	stp_identify_func	sc_identify;
	stp_getmode_func	sc_getmode;
	stp_setmode_func	sc_setmode;

	/* bindings */
	u_int			sc_tap_anywhere;/* button no; 0=none */
	u_int			sc_tap_lt, sc_tap_rt, sc_tap_lb, sc_tap_rb;
	int			sc_zaxis_t, sc_zaxis_b; /* 0 or 1 */
	int			sc_zaxis_l, sc_zaxis_r;
	int			sc_dnd;	/* 0 or 1 */
};

#ifdef _KERNEL
int	stp_init __P((void *));
void	stp_input __P((void *, struct stp_abspacket *));
#endif
